#!/bin/bash
#
# This scripts dumps the cn=config configuration

# # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # #
function usage() {
cat <<-USAGE
    Usage: $0 -f <filename> -g <git url> -k <ssh key file> -t [temp dir] -d -h
    -f <filename> file for the dump, defaults to cn-config.ldif
    -g "<git repo>" git repo to save the file to
    -k <ssh key file> location of the private key for the git repo
    -t [tempdir] optional tempdir to write stuff to, default /tmp
    -d If you want debugging messages too
    -h This usage message
USAGE
}
# # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # #

# Default debug to 0
debug=0

# Default dump file
phile='cn-config.ldif'
tempdir='/tmp'

# getops: define them here, and put in usage.
while getopts f:g:k:dh OPT; do
    case "$OPT" in
        h) usage
           exit 0                               ;;
        f) phile="$OPTARG"                      ;;
        g) git_url="$OPTARG"                    ;;
        k) ssh_keyfile="$OPTARG"                ;;
        t) tempdir="$OPTARG"                    ;;
        d) debug=1				;;
        *) echo "Unrecognized option: $OPT" >&2
           echo >&2
           usage
           exit 126                             ;;
    esac
done

# # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # #
# Add a debug function here
function debugecho() {
    if [ $debug ] ; then
        echo "debug: $*"
    fi
}
# # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # ## # # #

## >> Option parsing <<
# Check for presence of required options.
if [[ "$git_url" == "" ]] ; then
  usage
  debugecho "git URL is required"
  exit 1
fi

if [[ "${ssh_keyfile}" == "" ]] ; then
  usage
  debugecho "ssh keyfile is required"
  exit 1
elif [[ ! -f $ssh_keyfile ]] ; then
  usage
  debugecho "ssh keyfile '${ssh_keyfile}' not found"
  exit 1
fi

if [[ ! -d $tempdir ]] ; then
  usage
  debugecho "tempdir must exist"
  exit 1
fi
## >> End of Option parsing <<

## >> MAIN section <<

# 1. Dump the cn=conig tree to a temporary file.
# 2. Take out a 'modifiersName' attribute that causes a problem when
#    re-loading the cn=config.
# 3. Clone the Git repository where we track changes to cn-config.

### Step 1. Dump cn-config tree.
dumpfile="${tempdir}/${phile}"

# Remove the file if it already exists
if [[ -e $dumpfile ]] ; then
  rm $dumpfile
  debugecho "${dumpfile} was there, removed it"
fi

## Do the dump
this_date=$(date +%Y%m%d%H%M%S)
debugecho "running at ${this_date}"

debugecho "Writing to ${dumpfile}"

### Step 2. Remove an attribute from the dump that causes problems when re-loading.
# The sed is to eliminate that "proxied attribute" mess
output=$(slapcat -F /etc/ldap/slapd.d -b cn=config | sed -e "s/modifiersName:/#modifiersName:/" > $dumpfile )

if [[ ! -e $dumpfile ]] ; then
  debugecho "something went wrong with the slapcat, quitting"
  exit 1
else
  debugecho "config file is in ${dumpfile}"
fi

### 3. Clone the Git repository where we track changes to cn-config.
# Get the git dir from the git url - ASSUMES FLAT HIERARCHY (will have to change)
git_base=$( echo $git_url | awk -F: '{ print $2 }' | xargs basename | sed -e 's/.git//' )
debugecho "git dir base is ${git_base}"

git_dir="${tempdir}/${git_base}"
debugecho "git dir will be ${git_dir}"

# Delete git_dir (cannot clone into an existing directory).
if [[ -d $git_dir ]] ; then
  rm -rf $git_dir
  debugecho "deleted git directory ${git_dir}"
fi

# git clone
ssh-agent bash -c "ssh-add $ssh_keyfile; git clone $git_url ${git_dir}"
clone_err=$?

if (( $clone_err != 0 )) ; then
  debugecho "git clone failed: $clone_err"
  exit 5
fi

### 4. Copy the just-dumped cn-config to the Git repository working
###    directory, commit it, and push it.
debugecho "Copying to ${git_dir}"
cp $dumpfile $git_dir

cd $git_dir

# git commit
git add .
this_host=`hostname`
message="commit from ${this_host} on ${this_date}"
debugecho "git commit with message '$message'"

git -c user.name=auto-commit -c user.email=root@root.$thisHost commit -m "${message}"
commit_err=$?

if (( $commit_err != 0 )) ; then
  debugecho "couldn't commit ${commit_err}"
  exit 2
else
  debugecho "commit succeeded"
fi

# git push
ssh-agent bash -c "ssh-add $ssh_keyfile; git push origin master"
pusherr=$?

if [[ $pusherr ]] ; then
  debugecho "couldn't push to upstream repo"
  exit 3
fi

# Cleanup.
rm -rf $git_dir

debugecho "Done at `date`"
exit 0

############################################################################

# Documentation.  Use a hack to hide this from the shell.  Because of the
# above exit line, this should never be executed.
DOCS=<<__END_OF_DOCS__

=head1 NAME

ldap-config-to-git - Export cn=config tree to a git repository

=head1 SYNOPSIS

B<ldap-config-to-git> [-h] [-d] [-q] -g <git_url> -k <ssh key file> [-f <filename>]  [-t temp_dir]

=head1 DESCRIPTION

B<ldap-config-to-git> dumps the local cn=config tree into a file, copies
that file into a Git repository, commits that file, and then pushes the
git repository.

=head1 OPTIONS

=over

=item C<-g git_url> A Git URL that contains the repository for the changes
to this cn=config. Typically, something like
C<git@code.stanford.edu:authnz/ldap-replica-config/sbx.git>.

=item C<-k key_file> The path to an SSH private-key that has the rights to
read and write the repository specificed by the C<-g> option.

=item C<-f filename> The name of the file that gets dumped and
committed. It is  not a good idea to change this from the default
of C<cn-config.ldif>.

=item C<-t temp_dir> The temporary directory to do the cloning. Defaults
to C</tmp>.

=back


=head1 AUTHOR

Linda Laubenhimer <ljlgeek@stanford.edu> and Adam Lewenberg
<adamhl@stanford.edu>


=head1 SEE ALSO

git(1)

=cut

__END_OF_DOCS__
